@@ -1,12 +1,13 @@
 from typing import Any
 from graia.broadcast.entities.event import BaseEvent, BaseDispatcher
 from graia.broadcast.interfaces.dispatcher import DispatcherInterface
+from pydantic.main import BaseModel
 
-class ApplicationLaunched(BaseEvent):
-    app: "GraiaMiraiApplication"
+class ApplicationLaunched(BaseModel, BaseEvent):
+    app: Any
 
     def __init__(self, app) -> None:
-        self.app = app
+        super().__init__(app=app)
 
     class Dispatcher(BaseDispatcher):
         @staticmethod
@@ -15,11 +16,11 @@ class ApplicationLaunched(BaseEvent):
             if interface.annotation is GraiaMiraiApplication:
                 return interface.event.app
 
-class ApplicationLaunchedBlocking(BaseEvent):
+class ApplicationLaunchedBlocking(BaseModel, BaseEvent):
     app: Any
 
     def __init__(self, app) -> None:
-        self.app = app
+        super().__init__(app=app)
 
     class Dispatcher(BaseDispatcher):
         @staticmethod
@@ -28,15 +29,16 @@ class ApplicationLaunchedBlocking(BaseEvent):
             if interface.annotation is GraiaMiraiApplication:
                 return interface.event.app
 
-class ApplicationShutdowned(BaseEvent):
+class ApplicationShutdowned(BaseModel, BaseEvent):
     app: Any
 
     def __init__(self, app) -> None:
-        self.app = app
+        super().__init__(app=app)
 
     class Dispatcher(BaseDispatcher):
         @staticmethod
         def catch(interface: "DispatcherInterface"):
             from .. import GraiaMiraiApplication
             if interface.annotation is GraiaMiraiApplication:
-                return interface.event.app
\ No newline at end of file
+                return interface.event.app
+
