@@ -9,6 +9,7 @@ from .elements import ExternalElement, InternalElement, Element
 import regex
 import copy
 
+
 T = Element
 MessageIndex = Tuple[int, Optional[int]]
 
@@ -213,7 +214,7 @@ class MessageChain(BaseModel):
         Returns:
             List[T]: 获取到的符合要求的所有消息元素; 另: 可能是空列表([]).
         """
-        return [i for i in self.__root__ if isinstance(i, element_class)]
+        return [i for i in self.__root__ if type(i) is element_class]
 
     def asDisplay(self) -> str:
         """获取以字符串形式表示的消息链, 且趋于通常你见到的样子.
@@ -305,12 +306,21 @@ class MessageChain(BaseModel):
         return MessageChain.create(result)
 
     def asSerializationString(self) -> str:
-        """将消息链对象转为以 "Mirai 码" 表示特殊对象的字符串
+        """将消息链对象转为以 "Mirai 码" 表示特殊对象的字符串. 为了保证可逆，纯文本中的'['用'[_'替代
 
         Returns:
             str: 以 "Mirai 码" 表示特殊对象的字符串
         """
-        return "".join([i.asSerializationString() for i in self.__root__])
+        from .elements.internal import Plain
+        result = []
+        for e in self.__root__:
+            if isinstance(e, Plain):
+                result.append(e.asSerializationString().replace('[', '[_'))
+            else:
+                result.append(e.asSerializationString())
+        return ''.join(result)
+    
+    
     
     @classmethod
     def fromSerializationString(cls, string: str) -> "MessageChain":
@@ -319,35 +329,26 @@ class MessageChain(BaseModel):
         Returns:
             MessageChain: 转换后得到的消息链, 所包含的信息可能不完整.
         """
-        from .elements.internal import Source, Plain, AtAll, At, Face, Image, FlashImage
-        code_without_argument = {
-            "atall": AtAll
+        from .elements.internal import Plain, At, AtAll, Source, FlashImage, Image, Face
+        PARSE_FUNCTIONS = {
+            'atall': lambda args: AtAll(),
+            'source': lambda args: Source(id=args[0], time=args[1]),
+            'at': lambda args: At(target=args[0], display=args[1]),
+            'face': lambda args: Face(faceId=args[0]),
+            'image': lambda args: Image(imageId=args[0]),
+            'flash': lambda args: FlashImage(imageId=args[0]),
         }
-        origin_list = list(regex.finditer(
-            r"((?:\[mirai:(?P<name>[^:]+)\])|(?:\[mirai:(?P<name>[^\]]*)?:(?P<argument>.*?)?\]))|(?P<plaintext>.{1,})", string))
+        result = []
+        for match in regex.split(r'(\[mirai:.+?\])', string):
+            mirai = regex.fullmatch(r'\[mirai:(.+?)(:(.+?))\]', match)
+            if mirai:
+                # 容错：参数数量太少不行，太多可以
+                args = mirai.group(3).split(',')
+                result.append(PARSE_FUNCTIONS[mirai.group(1)](args))
+            elif match:
+                result.append(Plain(match.replace('[_', '[')))
+        return MessageChain.create(result)
 
-        result = [] # 默认为不可变型, 但这里要插入元素
-        for i in origin_list:
-            origin_pattern = i.groupdict()
-            if origin_pattern['name'] and not origin_pattern['argument']: # 无参数型
-                target_element = code_without_argument.get(origin_pattern['name'])
-                if target_element:
-                    result.append(target_element())
-            elif origin_pattern['name'] and origin_pattern['argument']: # 有参数
-                arguments = origin_pattern['argument'].split(",")
-                if origin_pattern['name'] == "source":
-                    result.append(Source(id=int(arguments[0]), time=arguments[1]))
-                elif origin_pattern['name'] == "at":
-                    result.append(At(target=int(arguments[0]), display=arguments[1]))
-                elif origin_pattern['name'] == "face":
-                    result.append(Face(faceId=int(arguments[0])))
-                elif origin_pattern['name'] == "image":
-                    result.append(Image(imageId=origin_pattern['argument']))
-                elif origin_pattern['name'] == "flash":
-                    result.append(FlashImage(imageId=origin_pattern['argument']))
-            elif origin_pattern['plaintext']:
-                result.append(Plain(origin_pattern['plaintext']))
-        return cls.create(tuple(result))
 
     def asMerged(self) -> "MessageChain":
         """合并相邻的 Plain 项, 并返回一个新的消息链实例
